from django.shortcuts import render, redirect, get_object_or_404
from django.contrib import messages
from django.utils import timezone
from django.contrib.auth.decorators import login_required
from django.core.exceptions import ObjectDoesNotExist
from django.core.paginator import Paginator
from django.http import HttpResponse, HttpResponseForbidden
from datetime import datetime
import csv

from account.models import Account
from .models import Campaign, Lead

@login_required
def create_campaign(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    accounts = Account.objects.all()
    if request.method == 'POST':
        try:
            name = request.POST.get('name')
            start_date = request.POST.get('start_date')
            end_date = request.POST.get('end_date')
            target_audience = request.POST.get('target_audience')
            channels = request.POST.get('channels')
            budget = request.POST.get('budget')
            objectives = request.POST.get('objectives')
            assigned_to_id = request.POST.get('assigned_to')
            assigned_to = Account.objects.get(id=assigned_to_id) if assigned_to_id else None
            assets = request.FILES.get('assets')

            if Campaign.objects.filter(name=name).exists():
                messages.error(request, 'A campaign with this name already exists.')
                return redirect('backoffice:campaign-list')

            if name and start_date and end_date and budget:
                start_date = timezone.make_aware(datetime.strptime(start_date, '%Y-%m-%d'))
                end_date = timezone.make_aware(datetime.strptime(end_date, '%Y-%m-%d'))
                campaign = Campaign.objects.create(
                    name=name,
                    start_date=start_date,
                    end_date=end_date,
                    target_audience=target_audience,
                    channels=channels,
                    budget=budget,
                    objectives=objectives,
                    assets=assets,
                    assigned_to=assigned_to
                )
                messages.success(request, 'Campaign Created Successfully!')
                return redirect('backoffice:campaign-list')
            else:
                messages.error(request, 'Please fill in all required fields.')
        except Exception as e:
            messages.error(request, f'An error occurred: {str(e)}')

    return render(request, 'backoffice/campaign/campaign.html', {'accounts': accounts})

@login_required
def campaign_list(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:
        accounts = Account.objects.all()
        search_query = request.GET.get('search_query', '')
        start_date = request.GET.get('start_date', '')
        end_date = request.GET.get('end_date', '')
        status = request.GET.get('status', '')  # Added status filter

        campaigns_query = Campaign.objects.all().order_by('-start_date')

        # Apply search filter
        if search_query:
            campaigns_query = campaigns_query.filter(name__icontains=search_query)

        # Apply date range filter
        if start_date and end_date:
            start_date_obj = datetime.strptime(start_date, "%Y-%m-%d").date()
            end_date_obj = datetime.strptime(end_date, "%Y-%m-%d").date()
            campaigns_query = campaigns_query.filter(start_date__gte=start_date_obj, end_date__lte=end_date_obj)

        # Apply status filter (if a status is provided)
        if status:
            campaigns_query = campaigns_query.filter(status=status)

        paginator = Paginator(campaigns_query, 10)
        page_number = request.GET.get('page')
        campaigns = paginator.get_page(page_number)

        context = {
            'campaigns': campaigns,
            'search_query': search_query,
            'start_date': start_date,
            'end_date': end_date,
            'status': status,  # Pass the selected status to the template
            'accounts': accounts
        }
        return render(request, 'backoffice/campaign/campaign.html', context)
    except Exception as e:
        messages.error(request, f'An error occurred: {str(e)}')
        return render(request, 'backoffice/campaign/campaign.html', {'error': str(e)})
 

@login_required
def update_campaign(request, campaign_id):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:
        # Fetch the campaign object or return a 404 if not found
        campaign = get_object_or_404(Campaign, id=campaign_id)

        if request.method == 'POST':
            # Retrieve form data from the POST request
            name = request.POST.get('name')
            start_date = request.POST.get('start_date')
            end_date = request.POST.get('end_date')
            target_audience = request.POST.get('target_audience')
            channels = request.POST.get('channels')
            budget = request.POST.get('budget')
            objectives = request.POST.get('objectives')
            assets = request.FILES.get('assets')

            # Retrieve interaction data
            clicks = request.POST.get('clicks')
            impressions = request.POST.get('impressions')
            conversions = request.POST.get('conversions')

            # Debug print for form data
            print(f"Form Data: name={name}, start_date={start_date}, end_date={end_date}, budget={budget}")

            # Validate required fields
            if name and start_date and end_date and budget:
                # Update campaign fields
                campaign.name = name
                campaign.start_date = start_date
                campaign.end_date = end_date
                campaign.target_audience = target_audience
                campaign.channels = channels
                campaign.budget = budget
                campaign.objectives = objectives

                # Handle file upload for assets
                if assets:
                    campaign.assets = assets

                # Handle interactions if provided
                if clicks and impressions and conversions:
                    # Ensure the campaign has an associated interaction record
                    interaction, created = Interaction.objects.get_or_create(campaign=campaign)
                    interaction.clicks = clicks
                    interaction.impressions = impressions
                    interaction.conversions = conversions
                    interaction.save()

                # Save the updated campaign
                campaign.save()

                # Success message
                messages.success(request, 'Campaign updated successfully.')
            else:
                messages.error(request, 'Please fill in all required fields.')

            return redirect('backoffice:campaign-list')

        context = {'campaign': campaign}
        return render(request, 'backoffice/campaign/campaign.html', context)

    except Exception as e:
        # Catch any unexpected errors and display them
        messages.error(request, f'An error occurred: {str(e)}')
        print(f"Error: {str(e)}")  # Debug print for error
        return redirect('backoffice:campaign-list')


@login_required
def delete_campaign(request, campaign_id):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:
        campaign = Campaign.objects.get(id=campaign_id)
        campaign.delete()
        messages.success(request, 'Campaign deleted successfully.')
    except ObjectDoesNotExist:
        messages.error(request, 'Campaign not found.')
    except Exception as e:
        messages.error(request, f'An error occurred: {str(e)}')

    return redirect('backoffice:campaign-list')

@login_required
def export_campaigns_excel(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:
        response = HttpResponse(content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename="campaigns.csv"'

        writer = csv.writer(response)
        writer.writerow(['Campaign Name', 'Start Date', 'End Date', 'Target Audience', 'Channels', 'Budget', 'Objectives'])

        campaigns = Campaign.objects.all().order_by('-start_date')
        for campaign in campaigns:
            writer.writerow([
                campaign.name,
                campaign.start_date.strftime('%Y-%m-%d'),
                campaign.end_date.strftime('%Y-%m-%d'),
                campaign.target_audience,
                campaign.channels,
                campaign.budget,
                campaign.objectives
            ])

        return response
    except Exception as e:
        messages.error(request, f'An error occurred while exporting campaigns: {str(e)}')
        return redirect('backoffice:campaign-list')

import logging

logger = logging.getLogger(__name__)

@login_required
def view_leads(request, campaign_id):
    logger.debug(f"Viewing leads for campaign {campaign_id}")
    if not (request.user.is_admin_user or request.user.is_super_user):
        logger.warning(f"Unauthorized access attempt by user {request.user}")
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:
        accounts = Account.objects.all()
        campaign = get_object_or_404(Campaign, id=campaign_id)
        leads = Lead.objects.filter(campaign=campaign)
        logger.info(f"Found {leads.count()} leads for campaign {campaign.name}")

        context = {
            'campaign': campaign,
            'leads': leads,
            'accounts': accounts
        }
        return render(request, 'backoffice/campaign/campaign_details.html', context)
    except Exception as e:
        logger.error(f"Error viewing leads for campaign {campaign_id}: {str(e)}")
        messages.error(request, f'An error occurred: {str(e)}')
        return render(request, 'backoffice/campaign/campaign_details.html', {'error': str(e)})

@login_required
def add_lead(request, campaign_id):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:
        accounts = Account.objects.all()
        campaign = get_object_or_404(Campaign, id=campaign_id)
        leads = Lead.objects.filter(campaign=campaign)

        if request.method == 'POST':
            assigned_to_id = request.POST.get('assigned_to')
            lead = Lead.objects.create(
                campaign=campaign,
                name=request.POST.get('name'),
                phone_number=request.POST.get('phone_number'),
                product_type=request.POST.get('product_type'),
                comments=request.POST.get('comments'),
                location=request.POST.get('location'),
                status=request.POST.get('status'),
                assigned_to=Account.objects.get(id=assigned_to_id) if assigned_to_id else None,
                next_follow_up_date=request.POST.get('next_follow_up_date')
            )
            messages.success(request, 'Lead added successfully.')
            return redirect('backoffice:view-leads', campaign_id=campaign_id)

        return render(request, 'backoffice/campaign/campaign_details.html', {
            'campaign': campaign,
            'leads': leads,
            'accounts': accounts
        })
    except Exception as e:
        messages.error(request, f'An error occurred: {str(e)}')
        return redirect('backoffice:campaign-list')

@login_required
def lead_list(request, campaign_id):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:
        accounts = Account.objects.all()
        campaign = get_object_or_404(Campaign, id=campaign_id)
        search_query = request.GET.get('search_query', '')
        start_date = request.GET.get('start_date', '')
        end_date = request.GET.get('end_date', '')

        leads_query = Lead.objects.filter(campaign=campaign).order_by('-created_at')

        if search_query:
            leads_query = leads_query.filter(phone_number__icontains=search_query)

        if start_date and end_date:
            start_date_obj = datetime.strptime(start_date, "%Y-%m-%d").date()
            end_date_obj = datetime.strptime(end_date, "%Y-%m-%d").date()
            leads_query = leads_query.filter(created_at__gte=start_date_obj, created_at__lte=end_date_obj)

        paginator = Paginator(leads_query, 10)
        page_number = request.GET.get('page')
        leads = paginator.get_page(page_number)

        return render(request, 'backoffice/campaign/campaign_details.html', {
            'campaign': campaign,
            'leads': leads,
            'search_query': search_query,
            'start_date': start_date,
            'end_date': end_date,
            'accounts': accounts,
        })
    except Exception as e:
        messages.error(request, f'An error occurred: {str(e)}')
        return redirect('backoffice:campaign-list')

@login_required
def update_lead(request, lead_id):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:
        lead = get_object_or_404(Lead, id=lead_id)

        if request.method == 'POST':
            lead.name = request.POST.get('name')
            lead.phone_number = request.POST.get('phone_number')
            lead.product_type = request.POST.get('product_type')
            lead.comments = request.POST.get('comments')
            lead.location = request.POST.get('location')
            lead.status = request.POST.get('status')
            next_follow_up_date_str = request.POST.get('next_follow_up_date')
            if next_follow_up_date_str:
                next_follow_up_date = datetime.strptime(next_follow_up_date_str, '%Y-%m-%d').date()
                lead.next_follow_up_date = next_follow_up_date
            else:
                lead.next_follow_up_date = None
            lead.save()

            messages.success(request, 'Lead updated successfully.')
            return redirect('backoffice:view-leads', campaign_id=lead.campaign.id)

        return render(request, 'backoffice/campaign/campaign_details.html', {'lead': lead})
    except Exception as e:
        messages.error(request, f'An error occurred: {str(e)}')
        return redirect('backoffice:campaign-list')

@login_required
def export_leads_excel(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:
        response = HttpResponse(content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename="leads.csv"'

        writer = csv.writer(response)
        writer.writerow(['Name', 'Phone Number', 'Product Type', 'Comments', 'Location', 'Status'])

        leads = Lead.objects.all()
        for lead in leads:
            writer.writerow([
                lead.name,
                lead.phone_number,
                lead.product_type,
                lead.comments,
                lead.location,
                lead.get_status_display()
            ])

        return response
    except Exception as e:
        messages.error(request, f'An error occurred while exporting leads: {str(e)}')
        return redirect('backoffice:campaign-list')

@login_required
def delete_lead(request, lead_id):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:
        lead = Lead.objects.get(id=lead_id)
        campaign_id = lead.campaign.id
        lead.delete()
        messages.success(request, 'Lead deleted successfully.')
        return redirect('backoffice:view-leads', campaign_id=campaign_id)
    except ObjectDoesNotExist:
        messages.error(request, 'Lead not found.')
    except Exception as e:
        messages.error(request, f'An error occurred: {str(e)}')

    return redirect('backoffice:campaign-list')